#!/usr/bin/perl5
#
# $Id: cat6.pl,v 1.3 1998/05/21 20:42:51 sheneman Exp $
#
##############################################################################
#
# The contents of this file are subject to the Netscape Public License Version
# 1.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.mozilla.org/NPL/ 
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
# the specific language governing rights and limitations under the License. 
#
# The Initial Developer of the Original Code is Netscape Communications
# Corporation. Portions created by Netscape are Copyright (C) 1998 Netscape
# Communications Corporation. All Rights Reserved.
#
##############################################################################
#
# cat6.pl
#
# This script attempts to generate a list of people who have switched their UID in
#   LDAP, but this did not get switched in PeopleSoft.
#
# AUTHOR:  Luke Sheneman
# HISTORY: 
# 
#   12/09/97 - First implementation
#
# (C)opyright 1997, Sheneman Script Factory, Inc
#

# modules that we need
use Ldapp;                      # LDAP module for Perl5, very cool!
use LdapConf;                   # Generic LDAP config foo
use LdapUtils;                  # Some defaults and utilities we use.

$| = 1; 			# flush all buffers immediately

# set some defaults here
$LDAP_SERVER                  = "hoth.mcom.com";
$LDAP_CERT_PATH               = "/etc/adduser_cert.db";

@ATTRIBUTES = ("uid", "manager", "employeetype", "employeenumber", "cn", "nscppersonexpdate", "userpassword", "telephonenumber", "objectclass");

# the path to the current files
$LEIF_FILE = "/u/sheneman/projects/cleanup/psoftsync_verbose.txt";
$PATH = "/u/sheneman/projects/cleanup/current";

# the names of the current files
$cat6_file      = "$PATH/cat6.txt";


# setup some LDAP parameters for every bind this session
sub setup_ldap {
    # set some defaults for the LDAP bind
    $ld{host} = $LDAP_SERVER;

    # set the cert path
    if($LDAP_CERT_PATH eq "") {
	$ld{port} = $LdapConf::PORT;
    } else {
	$ld{port} = $LdapConf::SSL_PORT;
    }
    $ld{root} = "$LdapConf::ROOT";
    $ld{cert} = "$LDAP_CERT_PATH";

    # return success
    return 0;
}


# do a linear search through the exceptions
sub check_in_peoplesoft {

    @results = `grep "No entry in PeopleSoft" $LEIF_FILE | cut -f4 -d" "`;

    $found = 1;   # start by assuming they are in PeopleSoft
    foreach $i (@results) {
	chop $i;  chop $i;

	# print("Comparing [$i] and [$uid]\n");
	if($i eq $uid) {
	    $found = 0;  # if we find them in Leif's output, they are not in PeopleSoft
	    last;
	}
    }

    return $found;
}

# determine if this LDAP account is a "pseudo-account"
sub is_pseudo { 

    $ispseudo = 0;
    foreach $obj (@objectclass) {
	if( ($obj eq "pseudoAccount") || ($obj eq "pseudoaccount") ) {
	    $ispseudo = 1;
	    last;
	}
    }

    return $ispseudo;
}

sub cat6_list {

    # establish the LDAP connection here
    $ld{conn} = new Ldapp($ld{host}, $ld{port}, $ld{bind}, $ld{pswd}, $ld{cert});
    if(!$ld{conn}) { 
	print("Error:  Could not connect to LDAP server $LDAP_SERVER\n");
	return -1;
    }

    open(OUT,    ">$cat6_file");

    # prime the file here
    print(OUT "\n");

    $entry = $ld{conn}->search($LdapConf::PEOPLE_ROOT, "onelevel", "uid=*", 0, @ATTRIBUTES);
    while($entry)
    {
	push(@entries, $entry);
	$entry = $ld{conn}->entry;
    }

    # generate a list of all people with the correct attributes
    foreach $entry (@entries) {
	if($entry) {

	    $uid              = @{$entry->{'uid'}}[0];
	    $manager          = @{$entry->{'manager'}}[0];
	    $employeetype     = @{$entry->{'employeetype'}}[0];
	    $employeenumber   = @{$entry->{'employeenumber'}}[0];
	    $cn               = @{$entry->{'cn'}}[0];
	    $exp              = @{$entry->{'nscppersonexpdate'}}[0];
	    $mailhost         = @{$entry->{'mailhost'}}[0];
	    $userpassword     = @{$entry->{'userpassword'}}[0];
	    $telephonenumber  = @{$entry->{'telephonenumber'}}[0];
	    @objectclass      = @{$entry->{'objectclass'}};

	    # see if we are a person or not
	    $ispseudo = &is_pseudo();

	    # assuming Lisa is still setting these
	    if( (($telephonenumber =~ "See") || ($telephonenumber =~ "see")) && (!$ispseudo)) {

		print(OUT "USER:         $uid\n");
		print(OUT "  FULL NAME:  $cn\n");
		print(OUT "  EMP TYPE:   $employeetype\n");
		print(OUT "  EMP NUMBER: $employeenumber\n");
		print(OUT "  MANAGER:    $manager\n");
		print(OUT "  MAILHOST:   $mailhost\n");
		print(OUT "  PASSWORD:   $userpassword\n");
		print(OUT "  EXPIRATION: $exp\n");
		print(OUT "\n\n");
	    }
	}
    }

    # close the LDAP connection here
    if($ld{conn}) {
	$ld{conn}->close;
    }

    close(OUT);

    return 0;
}


# setup LDAP connection information
&setup_ldap;


# MY LOGIC IN A NUTSHELL:
#
# iterate through all of the terminated employees
#   for each termed employee 
#     see if there is another active entry in LDAP with the same complete name
#       if so, see if that entry has a PeopleSoft counterpart.  If so, chances are
#          good that the user switched usernames and PeopleSoft did not switch at the same time.
&cat6_list;

exit 0;





